/*** DX11 Shader Manager class, Cristiaan Kop					 updated 27-12-2016 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::RENDERER (DX11)											***/
/*** header file for the CD3d ShaderMgr class interface				  				***/

#ifndef DX11SHADER_MGR_H
#define DX11SHADER_MGR_H

#include <d3d11.h>
#include <string>
#include <vector>
#include <atlbase.h>

#include "RENDERER/DX11/DX11shaderpack.h"


namespace Crealysm { namespace RENDERER
{

/**************************************************************************************/
/***								DX11 SHADER MGR CLASS							***/
/*** Manages the loading/ usage etc. of all shaders, including the VS inputlayouts	***/
/**************************************************************************************/

class DX11ShaderMgr 
{
public:
	DX11ShaderMgr(ID3D11Device *pDevice);
	~DX11ShaderMgr();

	DX11ShaderMgr(const DX11ShaderMgr& other) = delete;				// copy constructor: not allowed
	DX11ShaderMgr(DX11ShaderMgr&& other) = delete;					// move constructor: not allowed
	DX11ShaderMgr& operator=(const DX11ShaderMgr& other) = delete;	// copy assignment: not allowed
	DX11ShaderMgr& operator=(DX11ShaderMgr&& other) = delete;		// move assignment: not allowed

	uint GetNrShaderPacks()							const;
	uint GetNrInputLayouts()						const;

	uint GetCurrentPackId()							const;
	uint GetCurrentInputLayoutId()					const;

	const DX11ShaderPack* GetCurrentPackPtr()		const;
	ID3D11InputLayout* GetInputLayoutPtr()			const;

	DX11ShaderPack *AddShaderPack(const std::string &pFilename, const uint pPackId, const std::string &pTarget);

	bool LoadShaders(const std::string &pRootFolder, const std::string &pShaderConfigFile, const std::string &pTarget);

	bool SetPackById(const uint pId);
	bool SetPackByName(const std::string &pName);

private:
	bool CreateInputLayouts();
	
	CComPtr<ID3D11Device>					mDevice;

	DX11ShaderPack							mFallbackPack;
	std::vector<DX11ShaderPack>				mShaderPacks;

	std::vector<std::string>				mInputLayoutChecksums;		// MD5 HASH
	std::vector<CComPtr<ID3D11InputLayout>>	mInputLayouts;

	std::string								mShaderFolder;

	uint									mCurrentPackId;
	uint									mCurrentInputLayoutId;
};


}
}

#endif